@@ -5,11 +5,9 @@
  */
 
 /*  This file is ALSO:
- *  (C) Copyright David Abrahams 2001. Permission to copy, use,
- *  modify, sell and distribute this software is granted provided this
- *  copyright notice appears in all copies. This software is provided
- *  "as is" without express or implied warranty, and with no claim as
- *  to its suitability for any purpose.
+ *  Copyright 2001-2004 David Abrahams.
+ *  Distributed under the Boost Software License, Version 1.0.
+ *  (See accompanying file LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt)
  */
 
 # include "jam.h"
@@ -33,6 +31,7 @@
 # include <time.h>
 # include <assert.h>
 # include <string.h>
+# include <stdarg.h>
 
 /*
  * compile.c - compile parsed jam statements
@@ -1033,6 +1032,43 @@ evaluate_rule(
     return result;
 }
 
+/*
+ * Call the given rule with the specified parameters.
+ * The parameters should be of LIST* and end with NULL pointer.
+ * This differs from the 'evaluate_rule' in that frame
+ * for called rule is prepared in 'call_rule'.
+ *
+ * This function is usefull when builtin rule (in C) wants to
+ * call another rule, which might be implemented in Jam.
+ */
+LIST *call_rule( char *rulename, FRAME* caller_frame, ...)
+{
+    va_list va;
+    LIST *result;
+
+    FRAME       inner[1];
+    frame_init( inner );
+    inner->prev = caller_frame;
+    inner->module = caller_frame->module;
+    inner->procedure = 0;
+
+    va_start(va, caller_frame);    
+    for(;;)
+    {
+        LIST* l = va_arg(va, LIST*);
+        if (!l)
+            break;
+        lol_add(inner->args, l);
+    }
+    va_end(va);
+                
+    result = evaluate_rule(rulename, inner);    
+
+    frame_free(inner);
+
+    return result;
+}
+
 /*
  * compile_rules() - compile a chain of rules
  *
